/* @pjs pauseOnBlur=true; */
//=============================================================================
//  carte de champs (chp E, lignes de champ, equipotentielle, carte d'intensité) pour un dipole
//  ©J.Roussel -- 2 Nov 2015
//========== IDEES ==================

int OX1, OY1, OX2, OY2;//position des charges (q1>0, q2<0)
int a;//largeur du doublet
ArrayList lignes,equipot;
boolean afficheMagnitude=false;
boolean afficheControl=true;


void setup() {
  size(360,360);
  smooth();
  ellipseMode(CENTER);
  a=50;
  OX1=width/2;
  OX2=width/2;
  OY1=height/2-a;
  OY2=height/2+a;
  lignes = new ArrayList();
  equipot= new ArrayList();
}

void draw() {
  background(64); 
  drawChampE();//carte de champs
  //tracé des lignes de champ
  for (int i = lignes.size()-1; i >= 0; i--) { 
    LigneDeChamp LDC = (LigneDeChamp) lignes.get(i);
    LDC.display();
  }
  for (int i = equipot.size()-1; i >= 0; i--) { 
    Equipotentielle EQP = (Equipotentielle) equipot.get(i);
    EQP.display();
  }
  if (afficheMagnitude) {
    map_champE();
  }//carte d'intensité

  draw_charge();//affichage des charges
  if(afficheControl){affiche_control(0);}
}

void draw_charge() {
  fill(255);
  ellipse(OX1, OY1, 20, 20);
  ellipse(OX2, OY2, 20, 20);
  stroke(64);
  line(OX1-5,OY1,OX1+5,OY1);  
  line(OX1,OY1-5,OX1,OY1+5);
  line(OX2-5,OY2,OX2+5,OY2);
}

//champ vectoriel proportionnel au champ E (pour les lignes de champ)
// ATTENTION ! FAIRE&nbsp;: x=xx-OX et  y=yy-OY
PVector champ(float xx, float yy) {
  float p=(sq(xx-OX1)+sq(yy-OY1))/(sq(xx-OX2)+sq(yy-OY2));
  return new PVector((xx-OX1)-pow(p,1.5)*(xx-OX2),(yy-OY1)-pow(p,1.5)*(yy-OY2));
}

//Champ E(x,y) 
PVector champE(float xx, float yy) {
  float rr1, rr2;
  if ((dist(xx, yy, OX1, OY1)<10) || (dist(xx, yy, OX2, OY2)<10)) {
    return new PVector(0, 0);
  } else {
    rr1=pow(sq(xx-OX1)+sq(yy-OY1), 1.5);
    rr2=pow(sq(xx-OX2)+sq(yy-OY2), 1.5);
    return new PVector((xx-OX1)/rr1-(xx-OX2)/rr2, (yy-OY1)/rr1-(yy-OY2)/rr2);
  }
}



//Carte de champ
void drawChampE() {  
  strokeWeight(1);
  stroke(153);  
  float arrowsize = 4;
  float len=15;
  for (int i=0; i<width; i+=30)
    for (int j=0; j<height; j+=30) {
      if (champ(i, j).mag()!=0) {
        pushMatrix();// Translate to location to render vector
        translate(i, j);// Vector heading to get direction (pointing up is a heading of 0)
        rotate(champ(i, j).heading2D()); // Scale it to be bigger or smaller if necessary
        // Draw three lines to make an arrow
        line(0, 0, len, 0);
        line(len, 0, len-arrowsize, +arrowsize/2);
        line(len, 0, len-arrowsize, -arrowsize/2);
        popMatrix();
      }
    }
}

//tracé de la carte d'intensité en fausses couleurs. 
void map_champE() {  
  float z;
  color c1;
  strokeWeight(2);
  for (int i=0; i<width; i+=2) {
    for (int j=0; j<height; j+=2) {
      z=map(champE(i,j).mag(),0,0.001,0,1);
      c1=echelleCouleur(z);
      stroke(c1);
      point(i, j);
    }
  }
}


color echelleCouleur(float z) {
  float b;
  float R, G, B;
  if (z<=0.25) {
    b=0;
  } else if (z>=0.57) {
    b=1;
  } else {
    b=z/0.32-0.78125;
  }
  R=255*b;
  if (z<=0.42) {
    b=0;
  } else if (z>=0.92) {
    b=1;
  } else {
    b=2*z-0.84;
  }
  G=255*b;
  if (z<=0.42) {
    b=4*z;
  } else if (z>=0.92) {
    b=z/0.08-11.5;
  } else {
    b=-2*z+1.84;
  }
  B=255*b;
  return color(R, G, B,170);
}

color echelleCouleur2(float z) {
  float R, G, B;
  R=int(255*pow(z, 0.25));
  G=int(255*z*z*z);
  B=int(255*sin(2*PI*z));
  return color(R, G, B);
}

void affiche_control() {
   textSize(12);
   textAlign(LEFT,TOP);
   fill(255);
   textLeading(14);
   text("LEFT clic to draw a field line \nRIGHT clic to draw equipotential \nPress space bar to display intensity map \nPress R to reset\nPress I to hide panel",14,14);
 }


void resetA(){
	for (int i = lignes.size()-1; i >= 0; i--) {lignes.remove(i)};
}

void changeA(){
	afficheMagnitude=!afficheMagnitude;
}

void keyPressed() {
  if (key == ' ') {afficheMagnitude=!afficheMagnitude;}
  if (key == 'R' || key == 'r') {
  	for (int i = lignes.size()-1; i >= 0; i--) {lignes.remove(i)};
    for (int i = equipot.size()-1; i >= 0; i--) {equipot.remove(i);}   
	} 
  if (key == 'I' || key == 'i'){afficheControl=!afficheControl;} 
}


void mousePressed() {
  if (mouseButton == LEFT) {
    // une nouvelle ligne de champ est ajoutée à la liste (à la fin par défaut)
    lignes.add(new LigneDeChamp(mouseX, mouseY, 2));
  } 
  if (mouseButton == RIGHT) {
     equipot.add(new Equipotentielle(mouseX, mouseY, 4));
  }
}
//objet équipotentielle
class Equipotentielle {
  float x0;
  float y0;
  float pas;

  Equipotentielle(float posX, float posY, int ppas) {
    x0=posX;
    y0=posY;
    pas= ppas;
  }
  //méthode RK2
  void display() {
    strokeWeight(1);
    stroke(255,157,0);
    float xx1, yy1;
    float angle;
    int compteur=0;
    boolean stop=false;
    xx1=x0;
    yy1=y0;
    while (!stop) {
      angle=champ(xx1, yy1).heading2D()+HALF_PI;
      angle=champ(xx1+.5*pas*cos(angle), yy1+.5*pas*sin(angle)).heading2D()+HALF_PI;
      if (compteur%2==0) {
        line(xx1, yy1, xx1+pas*cos(angle), yy1+pas*sin(angle));
      }
      compteur++;
      xx1+=pas*cos(angle);
      yy1+=pas*sin(angle);
      if ((abs(xx1-OX1)<4 && abs(yy1-height/2)<75) || dist(OX1,OY1,xx1, yy1)>2*width) {
        stop=true;
      }
    }
    stop=false;
    xx1=x0;
    yy1=y0;
    while (!stop) {
      angle=champ(xx1, yy1).heading2D()+HALF_PI;
      angle=champ(xx1-.5*pas*cos(angle), yy1-.5*pas*sin(angle)).heading2D()+HALF_PI;
      if (compteur%2==0) {
        line(xx1, yy1, xx1-pas*cos(angle), yy1-pas*sin(angle));
      }
      compteur++;
      xx1-=pas*cos(angle);
      yy1-=pas*sin(angle);
      if ((abs(xx1-OX1)<4 && abs(yy1-height/2)<75) || dist(OX1,OY1,xx1, yy1)>2*width) {
        stop=true;
      }
    }
  }
}


// Objet ligne de champ
class LigneDeChamp {
  float x0;
  float y0;
  float pas;

  LigneDeChamp(float posX, float posY, int ppas) {
    x0=posX;
    y0=posY;
    pas= ppas;
  }


  //méthode RK2
  void display() {
    strokeWeight(2);
    stroke(0, 153, 204, 150);
    fill(0, 153, 204, 150);
    float xx1, yy1, xx2, yy2;
    float angle;
    xx1=x0;
    yy1=y0;
    xx2=x0;
    yy2=y0;
    pushMatrix();
    translate(xx1, yy1);
    rotate(champ(xx1, yy1).heading2D());
    beginShape(TRIANGLES);
    vertex(0, 3);
    vertex(8, 0);
    vertex(0, -3);
    endShape();
    rotate(-champ(xx1, yy1).heading2D());
    while (!StopTrace(xx1, yy1)) {
      angle=champ(xx1, yy1).heading2D();
      angle=champ(xx1+.5*pas*cos(angle), yy1+.5*pas*sin(angle)).heading2D();
      xx1+=pas*cos(angle);
      yy1+=pas*sin(angle);
      line(0, 0, pas*cos(angle), pas*sin(angle));
      translate(pas*cos(angle), pas*sin(angle));
    }
    popMatrix();
    pushMatrix();
    translate(xx2, yy2);
    while (!StopTrace(xx2, yy2)) {
      angle=champ(xx2, yy2).heading2D();
      angle=champ(xx2+.5*pas*cos(angle+PI), yy2+.5*pas*sin(angle+PI)).heading2D();
      xx2+=pas*cos(angle+PI);
      yy2+=pas*sin(angle+PI);
      line(0, 0, pas*cos(angle+PI), pas*sin(angle+PI));
      translate(pas*cos(angle+PI), pas*sin(angle+PI));
    }
    popMatrix();
  }

  Boolean StopTrace(float x, float y) {
    if ((dist(OX1, OY1, x, y)<10 || dist(OX1, OY1, x, y)>width || dist(OX2, OY2, x, y)<10 || dist(OX2, OY2, x, y)>width)) {
      return true;
    } else {
      return false;
    }
  }
}